# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later
"""Tests for the compare_vis script."""

import shutil

import pytest

from click.testing import CliRunner

from ptbench.scripts.compare_vis import _sorting_rule, compare_vis


@pytest.fixture()
def runner():
    return CliRunner()


@pytest.fixture
def compare_vis_dirs(temporary_basedir):
    input_dir = temporary_basedir / "compare_vis_input_folder"
    output_dir = temporary_basedir / "compare_vis_output_folder"
    if input_dir.exists():
        shutil.rmtree(input_dir)
    if output_dir.exists():
        shutil.rmtree(output_dir)
    input_dir.mkdir()
    output_dir.mkdir()
    return input_dir, output_dir


def test_sorting_rule():
    assert _sorting_rule("gradcam") == (0, "gradcam")
    assert _sorting_rule("gradcam/scorecam") == (1, "scorecam")
    assert _sorting_rule("fullgrad") == (2, "fullgrad")
    assert _sorting_rule("randomcam") == (4, "randomcam")
    assert _sorting_rule("test") == (3, "test")


def test_compare_vis_output_subdirectory(runner, compare_vis_dirs):
    input_dir, output_dir = compare_vis_dirs
    sub_output_dir = input_dir / "subfolder"
    sub_output_dir.mkdir()

    result = runner.invoke(
        compare_vis, ["-vv", "-i", str(input_dir), "-o", str(sub_output_dir)]
    )
    assert isinstance(result.exception, AssertionError)


def test_compare_vis_same_input_output(runner, compare_vis_dirs):
    input_dir, output_dir = compare_vis_dirs
    result = runner.invoke(
        compare_vis, ["-vv", "-i", str(input_dir), "-o", str(input_dir)]
    )
    assert isinstance(result.exception, AssertionError)


def test_compare_vis_no_subdirectories(runner, compare_vis_dirs):
    input_dir, output_dir = compare_vis_dirs
    result = runner.invoke(
        compare_vis, ["-vv", "-i", str(input_dir), "-o", str(output_dir)]
    )
    assert isinstance(result.exception, ValueError)
